#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/. 

import sys

import matplotlib.pyplot as plt
import numpy as np

import crunch_perf_results as c


R20_DATE_DEFAULT = '20140924'
R21_DATE_DEFAULT = '20141002'


def draw(app_name, r20, r21, show=True, save=False, save_prefix=''):
    plt.title(app_name)

    try:
        cr20 = r20['combined'][app_name][c.VC]
        med20 = cr20['stats']['median']
        p20 = np.percentile(cr20['durations'], 95)
        plt.hist(cr20['durations'], 32, label='2.0 (med=%d, p95=%d)' % (
            med20, p20), alpha=.25, color='blue')
        plt.xlabel('ms')
        plt.ylabel('# results')
        plt.axvline(x=med20, color='blue', linewidth=2, linestyle='-')
        plt.axvline(x=p20, color='blue', linewidth=2, linestyle='--')
    except:
        pass

    try:
        cr21 = r21['combined'][app_name][c.VC]
        med21 = cr21['stats']['median']
        p21 = np.percentile(cr21['durations'], 95)
        plt.hist(cr21['durations'], 32, label='2.1 (med=%d, p95=%d)' % (
            med21, p21), alpha=.25, color='red')
        plt.xlabel('ms')
        plt.ylabel('# results')
        plt.axvline(x=med21, color='red', linewidth=2, linestyle='-')
        plt.axvline(x=p21, color='red', linewidth=2, linestyle='--')
    except:
        pass

    plt.legend()

    if save:
        if save_prefix != '':
            save_prefix = save_prefix + '_'
        plt.savefig((save_prefix + app_name + '.png').replace('/', '_'))

    if show:
        plt.show()
    else:
        plt.close()


def get_app_names(r20, r21):
    names = r20['combined'].keys()
    names += r21['combined'].keys()
    names = list(set(names))
    names.sort()
    return names


def compare(r20_date=R20_DATE_DEFAULT, r21_date=R21_DATE_DEFAULT,
            show=True, save=False, save_prefix=None):
    if save_prefix is None:
        save_prefix = r21_date

    r20 = c.load_and_crunch_result_sets(
        '2.0-' + r20_date + '*json', test_pattern=c.VC, first_repetition=2)
    r21 = c.load_and_crunch_result_sets(
        '2.1-' + r21_date + '*json', test_pattern=c.VC, first_repetition=2)

    for name in get_app_names(r20, r21):
        draw(name, r20, r21, show, save, save_prefix)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        r20_date = sys.argv[1]
    else:
        r20_date = R20_DATE_DEFAULT

    if len(sys.argv) > 2:
        r21_date = sys.argv[2]
    else:
        r21_date = R21_DATE_DEFAULT

    compare(r20_date, r21_date, show=False, save=True)
